//+------------------------------------------------------------------+
//|                                                        Ticks.mq4 |
//|                                                       MetaQuotes |
//|                                           http://black-b0x.com   |
//|                    http://www.alpari-idc.ru/ru/experts/articles/ |
//+------------------------------------------------------------------+
#property copyright "BLACK-B0X.com"
#property link      "http://www.black-b0x.com"

#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Aqua
#property indicator_color2 Red
#property indicator_color3 Lime

extern int MaxDrawTicks       = 500;
extern int SpeedRectWidth     = 20;
extern int SpeedRectMargin    = 4;
extern int SpeedFontSize      = 10;
extern int SpeedTicks         = 5;
extern double SpeedScale      = 4;
extern bool UseMilliSeconds   = false;

double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];

static int _Bars = 0;
static int tickCounter = 0;

static int tickTime[];

#define SpeedRectName "SpeedRect"
#define SpeedTextName "SpeedText"

int init()
{
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0, ExtMapBuffer1);
   SetIndexEmptyValue(0,0.0);
   SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(1, ExtMapBuffer2);
   SetIndexEmptyValue(1,0.0);   
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2, ExtMapBuffer3);
   SetIndexEmptyValue(2,0.0);   
   
   for (int i = Bars - 1; i >= 0; i--)
   {
      ExtMapBuffer1[i] = 0.0;
      ExtMapBuffer2[i] = 0.0;
      ExtMapBuffer3[i] = 0.0;
   }

   _Bars = 0;
   tickCounter = 0;
   ArrayResize(tickTime, 0);

   return(0);
}




int deinit()
{
   switch (UninitializeReason())
   {
      case REASON_PARAMETERS:
         break;
      default:
         if (ObjectFind(SpeedRectName) >= 0) ObjectDelete(SpeedRectName);
         if (ObjectFind(SpeedTextName) >= 0) ObjectDelete(SpeedTextName);
         break;
   }
   return(0);
}

int start()
{
   Comment("FREE MT4 SYSTEMS AT:" + "\n" + "** black-b0x.COM **");
   
   if (isNewBar())
   {
      SetDelimeter();
   }
   else
   {
      tickCounter++;
      ShiftArray();
   }      

   ExtMapBuffer1[SpeedRectWidth + SpeedRectMargin] = Bid;      
   ExtMapBuffer2[SpeedRectWidth + SpeedRectMargin] = Ask;   
   ExtMapBuffer3[SpeedRectWidth + SpeedRectMargin] = Open[0];         

   SetSpeedRect();

   return(0);
}

bool isNewBar()
{
   if (_Bars != Bars)
   {
      _Bars = Bars;
      return(true);
   }   
   else return(false);
}

void SetDelimeter()
{
   if (_Bars != 0)
   {
      int handle = WindowFind(WindowExpertName());
      if (handle >= 0)
      {
         string objName = TimeToStr(Time[SpeedRectWidth]);
         if (ObjectCreate(objName, OBJ_VLINE, handle, Time[SpeedRectWidth + SpeedRectMargin], 0))
         {
            ObjectSet(objName, OBJPROP_COLOR, Yellow);
            ObjectSet(objName, OBJPROP_STYLE, STYLE_DASHDOT);
            ObjectsRedraw();           
         }
      }
   }
   return(0);
}




void ShiftArray()
{
   int d = SpeedRectWidth + SpeedRectMargin;

   if (tickCounter > (2 * MaxDrawTicks))
   {
      for (int i = tickCounter; i >= MaxDrawTicks; i--)
      {
         ExtMapBuffer1[d + i] = 0.0;
         ExtMapBuffer2[d + i] = 0.0;
         ExtMapBuffer3[d + i] = 0.0;            
      }
      tickCounter = MaxDrawTicks;
   }
   for (i = tickCounter; i > 0; i--)
   {
      ExtMapBuffer1[d + i] = ExtMapBuffer1[d + i - 1];
      ExtMapBuffer2[d + i] = ExtMapBuffer2[d + i - 1];
      ExtMapBuffer3[d + i] = ExtMapBuffer3[d + i - 1];      
   }
   for (i = 0; i < ObjectsTotal(); i++)
   {
      string objName = ObjectName(i); 
      if (ObjectType(objName) == OBJ_VLINE) 
      {
         int barTime = iBarShift(NULL, 0, ObjectGet(objName, OBJPROP_TIME1));
         ObjectSet(objName, OBJPROP_TIME1, Time[barTime + 1]); 
      }
   }
   return(0);
}

void SetSpeedRect()
{
   datetime time = 0;
   if (UseMilliSeconds) time = GetTickCount();
   else time = TimeCurrent();
   ArrayResize(tickTime, ArraySize(tickTime) + 1);
   tickTime[ArraySize(tickTime) - 1] = time;
   int k = ArraySize(tickTime) - SpeedTicks;
   if (k > 0)
   {
      for (int j = 0; j < SpeedTicks; j++)
      {
         tickTime[j] = tickTime[j + k];
      }
      ArrayResize(tickTime, SpeedTicks);
   }
   double speed = 0;
   int nbr = ArraySize(tickTime) - 1;
   if (nbr > 0)
   {
      datetime time0 = tickTime[0];
      if (time > time0 && SpeedScale > 0)
      {
         double dt = time - time0;
         if (UseMilliSeconds) dt/= 1000.;
         speed = (100. * nbr) / (SpeedScale * dt);
      }
      if (speed < 0) speed = 0;
   }

   double x = SpeedFontSize;
   int fontsize = (x / WindowBarsPerChart()) * 500.;
   if (fontsize < 8) fontsize = 8;
   if (fontsize > 16) fontsize = 16;

   int handle = WindowFind(WindowExpertName());
   if (handle >= 0)
   {
      double ymin = WindowPriceMin(handle);
      double ymax = WindowPriceMax(handle);
      int left = Time[SpeedRectWidth];
      int xmid = Time[SpeedRectWidth / 2];
      int right = Time[0];
      double bottom = ymin + (ymax - ymin) / 8;
      double top = bottom + (ymax - ymin) / 4;
      double ymid = (top + bottom) / 2 + fontsize * (top - bottom) / 30.;
      if (ObjectFind(SpeedRectName) == -1)
      {
         if (ObjectCreate(SpeedRectName, OBJ_RECTANGLE, handle, left, bottom, right, top))
         {
            ObjectSet(SpeedRectName, OBJPROP_STYLE, STYLE_SOLID);
            ObjectSet(SpeedRectName, OBJPROP_WIDTH, 5);
         }
      }
      else
      {
         ObjectSet(SpeedRectName, OBJPROP_TIME1, left);
         ObjectSet(SpeedRectName, OBJPROP_PRICE1, bottom);
         ObjectSet(SpeedRectName, OBJPROP_TIME2, right);
         ObjectSet(SpeedRectName, OBJPROP_PRICE2, top);
      }
      if (ObjectFind(SpeedRectName) != -1)
      {
         if (speed >= 50) ObjectSet(SpeedRectName, OBJPROP_COLOR, Green);
         else if (speed >= 25) ObjectSet(SpeedRectName, OBJPROP_COLOR, Blue);
         else if (speed >= 10) ObjectSet(SpeedRectName, OBJPROP_COLOR, Orange);
         else ObjectSet(SpeedRectName, OBJPROP_COLOR, Red);
      }
      if (ObjectFind(SpeedTextName) == -1)
      {
         ObjectCreate(SpeedTextName, OBJ_TEXT, handle, xmid, ymid);
      }
      else
      {
         ObjectSet(SpeedTextName, OBJPROP_TIME1, xmid);
         ObjectSet(SpeedTextName, OBJPROP_PRICE1, ymid);
      }
      if (ObjectFind(SpeedTextName) != -1)
      {
         string s = "" + DoubleToStr(speed, 0) + " %";
         ObjectSetText(SpeedTextName, s, fontsize, "Arial", White);
      }
   }
}

      

